@@ -229,8 +229,10 @@ struct ra_info *parseHeaders(struct ra_info *data)
 	int optlen;
 	struct nd_opt_hdr *hdr = (struct nd_opt_hdr *)(data->ra + 1);
 	int resid = data->ra_len - sizeof(struct nd_router_advert);
+	int prefixlbyt;
 
 	data->prefixes = apr_hash_make(data->pool);
+	data->routes = apr_hash_make(data->pool);
 
 	/* Iterate over the available option headers */
 	for (optlen = 0; resid > 0; resid -= optlen)
@@ -285,6 +287,7 @@ struct ra_info *parseHeaders(struct ra_info *data)
 					sizeof(struct in6_addr));
 				inet_ntop(AF_INET6, &(prf->prefix), prf->prefix_str, sizeof(prf->prefix_str));
 				prf->plen = p->nd_opt_pi_prefix_len;
+				prf->flags = p->nd_opt_pi_flags_reserved;
 
 				prf->index = data->number_prefixes;
 				data->number_prefixes++;
@@ -309,7 +312,35 @@ struct ra_info *parseHeaders(struct ra_info *data)
 			}
 			case ND_OPT_ROUTE_INFO:
 			{
-				/* We don't care about this */
+				struct nd_opt_route_info *p = 
+						(struct nd_opt_route_info *)hdr;
+				struct ra_route *rrt = apr_pcalloc(data->pool,
+								sizeof(struct ra_route));
+
+				/* Copy the data */
+				memset(&(rrt->prefix), 0, sizeof(struct in6_addr));
+				prefixlbyt = sizeof(struct in6_addr);
+				if (p->nd_opt_rti_prefixlen < (prefixlbyt*8))
+					prefixlbyt = (p->nd_opt_rti_prefixlen>>3);
+				if (((p->nd_opt_rti_prefixlen) % 8) != 0) prefixlbyt++;
+				if ((prefixlbyt % 8) != 0)
+				prefixlbyt = ((prefixlbyt>>3)+1)*8;
+				if (prefixlbyt > sizeof(struct in6_addr)) prefixlbyt = sizeof(struct in6_addr);
+				memcpy(&(rrt->prefix), 
+					((u08 *)p + sizeof(struct nd_opt_route_info)), 
+					prefixlbyt);
+				inet_ntop(AF_INET6, &(rrt->prefix), rrt->prefix_str, sizeof(rrt->prefix_str));
+				rrt->plen = p->nd_opt_rti_prefixlen;
+				rrt->preference = ((p->nd_opt_rti_flags & ND_RA_FLAG_RTPREF_MASK) >> 3);
+
+				rrt->index = data->number_routes;
+				data->number_routes++;
+
+				/* Add the route */
+				apr_hash_set(data->routes,
+					&(rrt->index),
+					sizeof(rrt->index),
+					rrt);
 				break;
 			}
 			default:
@@ -581,9 +612,10 @@ void executeActions(struct rule *this_rule, struct ra_info *data)
 void rafixd_clearRoute(struct ra_info *data)
 {
 	int i;
+	u08 hashi;
 	pcap_t *fd;
 	u08 *sendbuf = NULL;
-	u32 lhlen, sendlen, slen;
+	u32 lhlen, sendlen, slen, ralen;
 	char ifname[IF_NAMESIZE];
 	register u32 cksum = 0;
 	struct ether_header *ether;
@@ -591,6 +623,8 @@ void rafixd_clearRoute(struct ra_info *data)
 	struct nd_router_advert *ra;
 	struct ip6_pseudo_header phdr;
 	char errbuf[PCAP_ERRBUF_SIZE];
+	u08 *options=NULL, *new_options, *optins;
+	int options_len=0, options_len_inc;
 
 	memset(&errbuf, 0, sizeof(errbuf));
 	if_indextoname(data->interface, (char *)&ifname);
@@ -603,15 +637,83 @@ void rafixd_clearRoute(struct ra_info *data)
 		return;
 	}
 
+        /* construct zero-lifetime prefix and route info options */
+	for(hashi = 0 ; hashi < data->number_prefixes; hashi++)
+	{
+		struct ra_prefix *prf = 
+			apr_hash_get(data->prefixes, &hashi, sizeof(hashi));
+		struct nd_opt_prefix_info *hdr;
+		options_len_inc = sizeof(struct nd_opt_prefix_info);
+		new_options = realloc(options, options_len + options_len_inc);
+		if (new_options == NULL)
+		{
+			LOG( RAMOND_CRITICAL "failed to allocate"
+				" buffer sized '%lu'bytes\n", options_len + options_len_inc);
+			if (options != NULL) free(options);
+			pcap_close(fd);
+			return;
+		}
+		options = new_options;
+ 		hdr = (struct nd_opt_prefix_info *)(options + options_len); 
+		options_len += options_len_inc;
+		hdr->nd_opt_pi_type = ND_OPT_PREFIX_INFORMATION;
+		hdr->nd_opt_pi_len = options_len_inc/8;
+		hdr->nd_opt_pi_prefix_len = prf->plen;
+		hdr->nd_opt_pi_flags_reserved = prf->flags;
+
+		/* 2 hours in seconds, read RFC4862 section e 2 */
+		hdr->nd_opt_pi_valid_time = 7200;
+		hdr->nd_opt_pi_preferred_time = 0;
+		hdr->nd_opt_pi_reserved2 = 0;
+		memcpy(&(hdr->nd_opt_pi_prefix), &(prf->prefix),
+					sizeof(struct in6_addr));
+	}
+	for(hashi = 0 ; hashi < data->number_routes; hashi++)
+	{
+		struct ra_route *rrt = 
+			apr_hash_get(data->routes, &hashi, sizeof(hashi));
+		struct nd_opt_route_info *hdr = (struct nd_opt_route_info *)(options + options_len); 
+		int prefixlen = rrt->plen;
+		int prefixlbyt = (prefixlen>>3);
+		if ((prefixlen % 8) != 0) prefixlbyt++;
+		if ((prefixlbyt % 8) != 0)
+			prefixlbyt = ((prefixlbyt>>3)+1)*8;
+		if (prefixlbyt > sizeof(struct in6_addr)) prefixlbyt = sizeof(struct in6_addr);
+
+		options_len_inc = (sizeof(struct nd_opt_route_info) + prefixlbyt);
+		new_options = realloc(options, options_len + options_len_inc);
+		if (new_options == NULL)
+		{
+			LOG( RAMOND_CRITICAL "failed to allocate"
+				" buffer sized '%lu'bytes\n", options_len + options_len_inc);
+			if (options != NULL) free(options);
+			pcap_close(fd);
+			return;
+		}
+		options = new_options;
+ 		hdr = (struct nd_opt_route_info *)(options + options_len); 
+		options_len += options_len_inc;
+		hdr->nd_opt_rti_type = ND_OPT_ROUTE_INFO;
+		hdr->nd_opt_rti_len = options_len_inc/8;
+		hdr->nd_opt_rti_prefixlen = rrt->plen;
+		/* Preference as in the incoming RA */
+		hdr->nd_opt_rti_flags = (rrt->preference << 3);
+		hdr->nd_opt_rti_lifetime = 0;
+		memcpy(((u08 *)hdr + sizeof(struct nd_opt_route_info)), 
+                       &(rrt->prefix),
+			prefixlbyt);
+	}
+
 	lhlen = sizeof(struct ether_header);
-	sendlen = lhlen + sizeof(struct ip6_hdr) +
-			sizeof(struct nd_router_advert);
+	ralen = sizeof(struct nd_router_advert) + options_len;
+	sendlen = lhlen + sizeof(struct ip6_hdr) + ralen;
 
 	sendbuf = calloc(sendlen, 1);
 	if(sendbuf == NULL)
 	{
 		LOG( RAMOND_CRITICAL "failed to allocate"
 				" buffer sized '%lu'bytes\n", sendlen);
+		if (options != NULL) free(options);
 		pcap_close(fd);
 		return;
 	}
@@ -619,11 +721,12 @@ void rafixd_clearRoute(struct ra_info *data)
 	/* construct the IPv6 header */
 	ip6 = (struct ip6_hdr *)(sendbuf + lhlen);
 	ip6->ip6_flow = htonl(0x60000000); /* ip6->ip6_vfc = IPV6_VERSION << 4; */
+
 	ip6->ip6_nxt = IPPROTO_ICMPV6;
 	ip6->ip6_hlim = 255;
 	memcpy(&(ip6->ip6_src), &(data->from), sizeof(struct in6_addr));
 	i = inet_pton(AF_INET6, "ff02::1", &(ip6->ip6_dst));
-	ip6->ip6_plen = htons(sizeof(struct nd_router_advert));
+	ip6->ip6_plen = htons(ralen);
 
 	/* construct the RA header */
 	ra = (struct nd_router_advert *)(ip6 + 1);
@@ -681,15 +784,23 @@ void rafixd_clearRoute(struct ra_info *data)
 		i++;
 	}
 
+	/* Insert options field */
+	if (options_len > 0)
+	{
+		optins = (u08 *)(ra + 1);
+		memcpy(optins, options, options_len);
+	}
+	if (options != NULL) free(options);
+
 	i = 0;
-	while((2*i) < sizeof(struct nd_router_advert))
+	while((2*i) < ralen)
 	{
 		cksum += (u16) ((u16 *)ra)[i];
 		i++;
 	}
 
 	/*  Add left-over byte, if any */
-	if((2*i - sizeof(struct nd_router_advert)) > 0)
+	if((2*i - ralen) > 0)
 	{
 		cksum += (u08) ((u08 *)ra)[(2*i)+1];
 	}
@@ -726,6 +837,7 @@ int main(int argc, char *argv[])
 	}
 
 	apr_initialize();
+	atexit(apr_terminate);
 	apr_pool_create(&masterPool, NULL);
 
 	signal(SIGCHLD, sigchld_handler);
